limit = 5;

$( document ).ready(async function() {
    await RequestListModule.load_templates();
    await RequestListModule.prepare();
    RequestListModule.setup_pagination();
    RequestListModule.load_requests();
});

RequestListModule = ( function() {
    let old_offset = 0;
    let templates = {};
    let event_types = {};
    let groups_with_assign_pem = [];
    let prepare = async function(){
        groups_with_assign_pem = await get_group_caller_has_asign_permissions();
        event_types = await load_event_types();
    };
    let load_templates = async function(){
        const em_request_card = $.get("/templates/em_request_card.hbs");
        const pagination = $.get("/templates/pagination.hbs");
        const search = $.get("/templates/search.hbs");
        await Promise.all([em_request_card, pagination, search]).then(function(res){
            templates.em_request_card = Handlebars.compile(res[0]);
            templates.pagination = Handlebars.compile(res[1]);
            templates.search = res[2];
        });
        Handlebars.registerPartial('search', templates.search);
    };
    let setup_pagination = function(){
        pag = new Pagination("em_eventrequests_pagination", templates.pagination, ".requestpag", limit, load_requests);
    };
    let load_requests = function(offset, args){
        if(offset === undefined || !Number.isInteger(offset)){
            offset = 0;
        }
        old_offset = offset;

        if(args === undefined){
            args = "";
        }

        $.ajax({
            type: "GET",
            url: "/api/event_requests?limit=" + limit + "&offset=" + offset+args,
            contentType: 'application/json',
            timeout: 3000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!");
            },
            success: async function (data) {
                if (is_ok(data)) {
                    console.log(data);
                    $("#requestlist_accordion").html("");

                    let loading_queue = [];
                    $(data.requests).each(function(){
                        loading_queue.push(load_request(this))
                    });

                    async function load_request(request){
                        if(request.etype){
                            let types = [];

                            $(event_types).each(function(){
                                let type = {};
                                type.type_id = this.type_id;
                                type.name = this.name;
                                type.description = this.description;
                                type.is_billable = this.is_billable;

                                if(type.type_id === request.etype){
                                    type.active = true; //mark etype for request as active
                                }else{
                                    type.active = false;
                                }
                                types.push(type);
                            });

                            request.etypes = types;
                        }
                        if(request.organiser_id){
                            request.organiser = await get_organiser(request.organiser_id);
                        }
                        if(request.risk_potential){
                            if(request.risk_potential === 1){
                                request.risk_potential_low = true;
                            }   else if(request.risk_potential === 2){
                                request.risk_potential_medium = true;
                            }else if(request.risk_potential === 3){
                                request.risk_potential_high = true;
                            }
                        }

                        if(request.state !== 0){
                            if(request.state === 1){
                                request.state_1 = true;
                            }
                            if(request.state === 3){
                                request.state_3 = true;
                            }
                            if(request.state === 5){
                                request.enable_assignment = true;
                                request.state_5 = true;
                            }
                            if(request.state === 7){
                                request.enable_assignment = true;
                                request.state_7 = true;
                            }
                        }

                        let date = new Date(request.received);
                        request.received = ('0' + date.getDate()).slice(-2) + '.' + ('0' + (date.getMonth()+1)).slice(-2) + '.' + date.getFullYear() + ' ' + ('0' + date.getHours()).slice(-2) + ':' + ('0' + date.getMinutes()).slice(-2);

                        request.osearch_base = request.entity_id+"_organiser_search";

                        let gwap = [];
                        $(groups_with_assign_pem).each(function(){
                            let temp = {};
                            temp.id = this.id;
                            temp.name = this.name;
                            temp.description = this.description;

                            if(this.id === request.related_group){
                                temp.active = true;
                            }else{
                                temp.active = false;
                            }
                            gwap.push(temp);
                        });
                        request.groups_with_assign_permission = gwap;

                        return request;
                    }

                    await $.when.apply($, loading_queue).then(function(){
                        var objects = arguments;

                        $(objects).each(function(){
                            $("#requestlist_accordion").append(templates.em_request_card(this));
                            add_organiser_search(this);
                        })
                    })

                    $(".edit_request_button").off("click").on("click", enable_edit);
                    $(".delete_request_button").off("click").on("click", delete_listener);
                    pag.render(data.total_request_count, offset);
                }
            }
        });
    };
    let get_group_caller_has_asign_permissions = async function(){
        let res = [];

        let groups_caller_has_assign_permissions = await get_caller_permission_context("modules.event_management.requests.assignments.assign");
        let groups = await load_groups_async();

        $(groups).each(function(){
            let group = this;
            if(groups_caller_has_assign_permissions.indexOf(group.group_id) !== -1){
                let temp = {};
                temp.name = group.name;
                temp.description = group.description;
                temp.id = group.group_id;
                res.push(temp);
            }
        })

        return res;
    }
    let enable_edit = function(){
        let card = $(this).closest('.card-body');
        card.find(".edit_request_button").text("Speichern").off("click").on("click", save_change);
        card.find(":input[readonly='readonly']").removeAttr("readonly").removeAttr("disabled");
    };
    let save_change = function(){
        let card = $(this).closest('.card-body');
        let entity_id = $(this).data("request-id");

        let name = $("#"+entity_id+"-name").val();
        let etype = $("#"+entity_id+"-etype").val();
        let timescale = $("#"+entity_id+"-timescale").val();
        let site = $("#"+entity_id+"-site").val();
        let risk_potential = $("#"+entity_id+"-risk_potential").val();
        let expected_attendees = $("#"+entity_id+"-expected_attendees").val();
        let specials = $("#"+entity_id+"-specials").val();
        let requested = $("#"+entity_id+"-requested").val();
        let organiser = $("#"+entity_id+"_organiser_search").data("entity-id");
        let contact_on_site_name = $("#"+entity_id+"-contact_on_site_name").val();
        let contact_on_site_phone = $("#"+entity_id+"-contact_on_site_phone").val();
        let state = $("#"+entity_id+"-state").val();
        let assignment = $("#"+entity_id+"-assignment").val();

        let er = {};
        er.entity_id = entity_id;
        if(name !== ""){
            er.name = name;
        }
        if(etype !== ""){
            er.etype = etype;
        }
        if(timescale !== ""){
            er.timescale = timescale;
        }
        if(site !== ""){
            er.site = site;
        }
        if(risk_potential !== ""){
            er.risk_potential = risk_potential;
        }
        if(expected_attendees !== "") {
            er.expected_attendees = expected_attendees;
        }
        if(specials !== "") {
            er.specials = specials;
        }
        if(requested !== "") {
            er.requested = requested;
        }
        if(organiser !== ""){
            er.organiser_id = organiser;
        }
        if(contact_on_site_name !== ""){
            er.contact_on_site_name = contact_on_site_name;
        }
        if(contact_on_site_phone !== ""){
            er.contact_on_site_phone = contact_on_site_phone;
        }
        if(state !== ""){
            er.state = state;
            if(state === "5" && assignment !== "" && assignment !== "none"){ //update state to request taken by group if related group is set and status is request published
                er.state = "7";
            }else{
                console.log("ass:"+assignment+"state:"+state);
            }
        }
        if(assignment !== "" && assignment !== "none" && assignment != null){
            er.related_group = assignment;
        }

        $.ajax({
            type: "PUT",
            url: "/api/event_requests/" + entity_id,
            data: JSON.stringify(er),
            contentType: 'application/json',
            timeout: 3000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!");
            },
            success: async function (data) {
                if (is_ok(data)) {
                    window.location.reload(true)
                    //card.find(".edit_request_button").text("Bearbeiten").off("click").on("click", enable_edit);
                    //card.find(":input").attr("readonly", true).attr("disabled", true);
                }
            }
        });
    };
    let add_organiser_search = function(request){
        let delete_callback = function(){

        };
        var val = request.organiser_id;
        var val_name = null;
        if(request.organiser){
            val_name = request.organiser.company;
            if(request.organiser.firstname && request.organiser.lastname){
                val_name += ": "+request.organiser.firstname+" "+request.organiser.lastname
            }
        }
        var organiser_search = new MiniSearchbar(request.entity_id+"_organiser_search", null, val, val_name, delete_callback);
        organiser_search.setup($("#"+this.entity_id+"_organiser_search"));
    };
    let delete_listener = function(){
        let request_id = $(this).data("request-id");
        let name = $(this).data("request-name");
        console.log(name);

        $("#request_delete_modal_request_name").html(name);
        $("#request_delete_modal_confirmation").val("").attr("placeholder", name);

        $("#request_delete_modal_delete_button").off("click").on("click", async function () {
            let input = $(this).closest(".modal-content").find("#request_delete_modal_confirmation");
            if (input.val() === input.attr("placeholder")) {
                await delete_request(request_id);
            }
        })

        $("#request_delete_modal").modal();
    };
    let delete_request = async function(entity_id){
        $.ajax({
            type: "DELETE",
            url: "/api/event_requests/" + entity_id,
            contentType: 'application/json',
            timeout: 3000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!");
            },
            success: async function (data) {
                if (is_ok(data)) {
                    alert("Erfolgreich gelöscht.");
                }
            }
        });
    }
    return{
        load_requests: load_requests,
        load_templates: load_templates,
        setup_pagination: setup_pagination,
        prepare: prepare,
        save_change: save_change,
    }
}());