let all_checked = false;
let all_members_checked = false;
let delete_list = [];
let active_group_id = "";

$( document ).ready(async function() {
    GroupModule.states = await GroupModule.load_states();
    await GroupModule.load_templates();
    $(".new_group_button").on("click", GroupModule.create_group_button_listener);
    $(".groups_delete_button").on("click", GroupModule.delete_group_button_listener);
    $(".check_all_groups").on("click",GroupModule.check_all_button_listener);

    $("#groups_delete_modal_confirmation").on("keyup",  GroupModule.delete_group_confirmation_check);
    $("#groups_delete_modal_confirmation").bind("paste", GroupModule.delete_group_confirmation_check);
    $("#groups_delete_modal_confirmation").bind("cut", GroupModule.delete_group_confirmation_check);

    $("#groups_delete_modal_delete_button").on("click", GroupModule.delete_groups_request);
    $(".group_list_row").on("click", GroupModule.load_group_detailed_view);

    $(".group-detailed-view-member-search-member-to-add").on("click", GroupModule.add_member_click_listener);
    $(".group_detailed_view_check_all_members").on("click", GroupModule.check_all_members_listener);
    $(".group-detailed-view-remove-member-button").on("click", GroupModule.remove_member_from_group_listener);
    $(".group_detailed_view_submit_core_data").on("click", GroupModule.save_core_data_button_listener);
});
//TODO: Remove add member to group in UI if user missing permission
GroupModule = ( function() {
    let states = {};
    let templates = {};
    let load_templates = async function(){
        const mm_groups_members_row = $.get("/templates/mm_group_members_row.hbs");
        const search = $.get("/templates/search.hbs");
        await Promise.all([mm_groups_members_row, search]).then(function(res){
            templates.mm_groups_members_row = Handlebars.compile(res[0]);
            templates.search = res[1];
        });
        Handlebars.registerPartial('search', templates.search);
    };
    let load_states = async function(){
        const res = await $.ajax({
            type: "GET",
            url: "/api/groups/states",
            contentType: 'application/json',
            timeout: 10000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!!");
            },
        });
        if (is_ok(res)) {
            return res;
        }
    };
    let load_group_detailed_view = function(){
        active_group_id = $(this).data("group-id");
        $(".group_detailed_view").prop("hidden", false);

        $.ajax({
            type: "GET",
            url: "/api/groups/"+active_group_id+"?detailed",
            contentType: 'application/json',
            timeout: 10000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!!");
            },
            success: function (data) {
                if(is_ok(data)) {
                    clear_detailed_view();
                    set_group_detailed_view_core_data(data.group.name, data.group.description, !data.caller_permissions.permission_groups_core_edit);
                    if(data.caller_permissions.permission_groups_members_view){
                        set_group_detailed_view_member_list(data.members)
                    }else{
                        $(".group_detailed_view_member_list").hide();
                    }
                }
            }
        });
    };
    let clear_detailed_view = function(){
        $(".group_detailed_view_member_tr").remove();
        $("#group_detailed_view_name_input").val("");
        $("#group_detailed_view_description_input").val("");
        $("#group_detailed_view_name_input").prop('readonly', false);
        $("#group_detailed_view_description_input").prop('readonly', false);
        $(".group_detailed_view_member_list").show();
        $(".group_detailed_view_submit_core_data").show();
    };
    let set_group_detailed_view_core_data = function(name, description, readonly){
      $("#group_detailed_view_name_input").val(name);
      $("#group_detailed_view_description_input").val(description);
      if(readonly){
          $("#group_detailed_view_name_input").prop('readonly', true);
          $("#group_detailed_view_description_input").prop('readonly', true);
          $(".group_detailed_view_submit_core_data").hide();
      }
    };
    let set_group_detailed_view_member_list = function(members){
        if(!members){
            return;
        }
        $(members).each(function(index, member){
            member.states = Object.assign({}, GroupModule.states);

            $.each(member.states, function(index, state){
                if(state.state_id === member.group_state){
                    state.active = true;
                }else{
                    state.active = false;
                }
            });

            $(".group_detailed_view_tbody").append(templates.mm_groups_members_row(member));

        });
        $(".group_detailed_view_member_state_select").off("change").on("change", update_group_member_state);
    };
    let update_group_member_state = function(){
        $.ajax({
            type: "PUT",
            url: "/api/groups/"+$(this).data("group-id")+"/members/"+$(this).data("member-id")+"/state/"+this.value,
            contentType: 'application/json',
            timeout: 10000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!!");
            },
            success: function (data) {
                if(is_ok(data)) {
                    show_saved_animation();
                }
            }
        });
    };
    let create_group_button_listener = function(){
        if($.trim($('#new_group_name').val()) == ''){
            alert("Fehler: Es wurde kein Gruppenname angegeben!");
            return
        }

        let create_group_data = {};
        let group_data = {};
        let role_permissions = [];

        group_data.group_name = $("#new_group_name").val();
        if(!$.trim($("#new_group_description").val()) == ''){
            group_data.group_description = $("#new_group_description").val();
        }

        $(".new_group_role_row").each(function(index, row){
            let group_role_perm = {};
            group_role_perm.role_id = $(row).data("role-id");
            group_role_perm.permission_groups_core_edit = $(row).find("input.permission_groups_core_edit").prop("checked");
            group_role_perm.permission_groups_delete = $(row).find("input.permission_groups_delete").prop("checked");
            group_role_perm.permission_groups_members_view = $(row).find("input.permission_groups_members_view").prop("checked");
            group_role_perm.permission_groups_members_edit = $(row).find("input.permission_groups_members_edit").prop("checked");
            group_role_perm.permission_groups_permissions_edit = $(row).find("input.permission_groups_permissions_edit").prop("checked");
            group_role_perm.permission_groups_permissions_view = $(row).find("input.permission_groups_permissions_view").prop("checked");

            role_permissions.push(group_role_perm);
        });

        create_group_data.role_permissions = role_permissions;
        create_group_data.group_data = group_data;

        create_group_request(create_group_data);
        };
    let create_group_request = function(data){
        $.ajax({
            type: "POST",
            url: "/api/groups",
            contentType: 'application/json',
            data: JSON.stringify(data),
            timeout: 10000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!!");
            },
            success: function (data) {
                if(is_ok(data)) {
                    location.reload();//TODO: update data directly instead of reloading page
                }
            }
        });
    };
    let delete_group_confirmation_check = function(){
        if($("#groups_delete_modal_confirmation").val() === "GRUPPEN LÖSCHEN"){
            $("#groups_delete_modal_delete_button").prop("disabled", false);
        }
    };
    let delete_group_button_listener = function(){
        delete_list = [];
        let delete_list_names = "";
        $(".group_entry_checkbox").each(function(){
            if($(this).prop('checked')) {
                delete_list.push($(this).data("group-id"));
                delete_list_names = delete_list_names + $(this).data("group-name")+", ";
            }
        })

        if(delete_list.length == 0){
            return;
        }

        $("#groups_delete_modal_delete_button").prop("disabled", true);
        $("#groups_delete_modal_group_list").html("");
        $("#groups_delete_modal_confirmation").val("");

        $("#groups_delete_modal_group_list").html(delete_list_names);
        $("#groups_delete_modal").modal();
    };
    let check_all_button_listener = function(){
        if(all_checked){
            $(".group_entry_checkbox").prop('checked', false);
            all_checked = false;
        }else {
            $(".group_entry_checkbox").prop('checked', true);
            all_checked = true;
        }
    };
    let delete_groups_request = function(){
        if(delete_list.length===0){
            return;
        }
        $.ajax({
            type: "DELETE",
            url: "/api/groups",
            contentType: 'application/json',
            data: JSON.stringify(delete_list),
            timeout: 10000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!!");
            },
            success: function (data) {
                if(is_ok(data)) {
                    location.reload(); //TODO: delete row in table instead of reloading page
                }
            }
        });
    };
    let add_member_click_listener = function(element){
        GroupModule.add_member_to_group(active_group_id, $(element).data("entity-id"), $(element).data("firstname"), $(element).data("lastname"));
    }
    let add_member_to_group = function(group_id, member_id, member_firstname, member_lastname){
        $.ajax({
            type: "PUT",
            url: "/api/groups/"+group_id+"/members/"+member_id,
            contentType: 'application/json',
            timeout: 10000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!!");
            },
            success: function (data) {
                if(is_ok(data)) {
                    let member = {};
                    member.entity_id = member_id;
                    member.group_id = group_id;
                    member.firstname = member_firstname;
                    member.lastname = member_lastname;
                    member.states = Object.assign({}, GroupModule.states);
                    $.each(member.states, function(index, state){
                        state.active = false;
                    });
                    $(".group_detailed_view_tbody").append(templates.mm_groups_members_row(member));
                    $(".group_detailed_view_member_state_select").off("change").on("change", update_group_member_state);
                }
            }
        });
    };
    let check_all_members_listener = function(){
        if(all_members_checked){
            $(".group_detailed_view_member_selected").prop('checked', false);
            all_members_checked = false;
        }else {
            $(".group_detailed_view_member_selected").prop('checked', true);
            all_members_checked = true;
        }
    };
    let remove_member_from_group_listener = function(){
        $(".group_detailed_view_member_selected").each(function(){
            let member_to_delete = $(this).data("member-id");
            if($(this).prop("checked")) {
                $.ajax({
                    type: "DELETE",
                    url: "/api/groups/"+active_group_id+"/members/"+member_to_delete,
                    contentType: 'application/json',
                    timeout: 10000,
                    error: function () {
                        alert("Es ist ein Fehler aufgetreten!!");
                    },
                    success: function (data) {
                        if(is_ok(data)) {
                        $(".group_detailed_view_member_tr").each(function(){
                           if($(this).data("member-id") === member_to_delete){
                               $(this).remove();
                           }
                        });
                        }
                    }
                });
            }
        })
    };
    let save_core_data_button_listener = function(){
        let group = {};
        group.group_name = $("#group_detailed_view_name_input").val();
        let description = $("#group_detailed_view_description_input").val();
        if(description!=null && description!==""){
            group.group_description = description;
        }

        $.ajax({
            type: "PUT",
            url: "/api/groups/"+active_group_id,
            contentType: 'application/json',
            data: JSON.stringify(group),
            timeout: 10000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!!");
            },
            success: function (data) {
                if(is_ok(data)) {
                   location.reload();
                }
            }
        });
    };
    return{
        create_group_button_listener: create_group_button_listener,
        delete_group_button_listener: delete_group_button_listener,
        check_all_button_listener: check_all_button_listener,
        delete_group_confirmation_check: delete_group_confirmation_check,
        delete_groups_request: delete_groups_request,
        load_group_detailed_view: load_group_detailed_view,
        add_member_to_group: add_member_to_group,
        add_member_click_listener: add_member_click_listener,
        check_all_members_listener: check_all_members_listener,
        remove_member_from_group_listener: remove_member_from_group_listener,
        save_core_data_button_listener: save_core_data_button_listener,
        load_states: load_states,
        states: states,
        load_templates: load_templates,
    };
})();