active_role = null;

$(document).ready(async function () {
    await SettingsRoleModule.load_templates();
    await SettingsRoleModule.load_role_list();
    SettingsRoleModule.prepare();
});

SettingsRoleModule = (function () {
    let templates = {};
    let roles_list = [];
    let load_templates = async function () {
        const role_list = $.get("/templates/settings_roles.hbs");
        const member_row = $.get("/templates/settings_roles_member_row.hbs");
        const search = $.get("/templates/search.hbs");

        await Promise.all([role_list, member_row, search]).then(function (res) {
            templates.role_list = Handlebars.compile(res[0]);
            templates.member_row = Handlebars.compile(res[1]);
            templates.search = res[2];
        });
        Handlebars.registerPartial('search', templates.search);
    };
    let prepare = function(){
        var member_search = new MiniSearchbar("role_member_search", add_member_to_role_listener, null, null, null);
        member_search.setup($("#role_member_search"));
        $(".role-detailed-view-remove-member-button").on("click", delete_members_listener);
        $(".role_detailed_view_submit_core_data").on("click", update_core_data_listener);
    };
    let add_member_to_role_listener = async function(res){
        let member_id = $(res).data("entity-id");
        let firstname = $(res).data("firstname");
        let lastname = $(res).data("lastname");

        if(is_ok(await add_member_to_role(active_role, member_id))){
            await load_role_list();
            $("tr").each(function(){
                if($(this).data("role-id") === active_role){
                    $(this).click();
                }
            })
        }
    };
    let add_member_to_role = async function(role, member){
        const res = $.ajax({
            type: "POST",
            url: "/api/roles/"+role+"/members/"+member,
            contentType: 'application/json',
            timeout: 3000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!");
            }
        });
        return res;
    };
    let load_roles = async function(){
        const res = $.ajax({
            type: "GET",
            url: "/api/roles",
            contentType: 'application/json',
            timeout: 3000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!");
            }
        });
        if (is_ok(res)) {
            return res;
        }
    };
    let load_role_members = async function(role){
        const res = $.ajax({
            type: "GET",
            url: "/api/roles/"+role+"/members",
            contentType: 'application/json',
            timeout: 3000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!");
            }
        });
        if (is_ok(res)) {
            return res;
        }
    };
    let load_role_list = async function(){
        let roles = await load_roles();
        let res = [];

        for(let i=0;i<roles.length;i++){
            let temp = {};
            temp.id = roles[i].id;
            temp.description = roles[i].description;
            temp.members = await load_role_members(roles[i].id);
            temp.member_count = temp.members.length;
            res.push(temp);
        }

        $("#roles").html(templates.role_list(res));
        roles_list = res;

        $(".role-tr").off("click").on("click", function(){
            let role_id = $(this).data("role-id");
            active_role = role_id;

            let role = null;

            for(let i=0;i<roles_list.length;i++){
                if(roles_list[i].id === role_id){
                    role = roles_list[i];
                    break;
                }
            }
            $("#role_detailed_view_name_input").val(role.id);
            $("#role_detailed_view_name_old").val(role.id);
            $("#role_detailed_view_description_input").val(role.description);

            $(".role_detailed_view_tbody").html(templates.member_row(role.members));
            $(".role_detailed_view").removeAttr('hidden');
        });
    };
    let delete_members_listener = async function(){
        let delete_list = [];
        $(".member_checkbox:checked").each(function(){
            delete_list.push($(this).data("member-id"));
        });

        let promises = [];

        $(delete_list).each(function(){
           promises.push(remove_member_from_role(active_role, this));
        });
        await Promise.all(promises).then(async function (res) {
            await load_role_list();
            $("tr").each(function(){
                if($(this).data("role-id") === active_role){
                    $(this).click();
                }
            })
        });
    };
    let remove_member_from_role = async function(role, member){
        const res = $.ajax({
            type: "DELETE",
            url: "/api/roles/"+role+"/members/"+member,
            contentType: 'application/json',
            timeout: 3000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!");
            }
        });
        return res;
    };
    let update_core_data_listener = async function(){
        let old_id = $("#role_detailed_view_name_old").val();
        let updated = {};
        updated.id = $("#role_detailed_view_name_input").val();
        updated.description = $("#role_detailed_view_description_input").val();
        if(is_ok(await update_core_data(old_id, updated))){
            await load_role_list();
            $("tr").each(function(){
                if($(this).data("role-id") === active_role){
                    $(this).click();
                }
            })
        }
    };
    let update_core_data = async function(old_id, role){
        const res = $.ajax({
            type: "PUT",
            url: "/api/roles/"+old_id,
            data: JSON.stringify(role),
            contentType: 'application/json',
            timeout: 3000,
            error: function () {
                alert("Es ist ein Fehler aufgetreten!");
            }
        });
        return res;
    };
    return {
        load_templates: load_templates,
        load_role_list: load_role_list,
        prepare: prepare,
    }
}());