use crate::helper::settings::Settings;
use rocket::State;
use crate::database::model::appointments::{AppointmentType, Appointment};
use crate::database::controller::connector::establish_connection;
use diesel::{RunQueryDsl, ExpressionMethods};
use chrono::NaiveDate;
use diesel::query_dsl::filter_dsl::FilterDsl;



pub fn get_appointment_types(settings: &State<Settings>) -> Result<Vec<AppointmentType>, diesel::result::Error>{
    use crate::schema::appointment_types::dsl::*;

    let connection = establish_connection(settings);

    match appointment_types.load(&connection){
        Ok(atype) => Ok(atype),
        Err(e) => {
            error!("Couldn't get appointment types: {}", e);
            Err(e)
        }
    }
}

pub fn add_appointment(settings: &State<Settings>, type_id2: uuid::Uuid, entity_id2: uuid::Uuid, appointment_date2: NaiveDate) -> Result<Appointment, diesel::result::Error>{
    use crate::schema::appointments::dsl::*;

    let connection = establish_connection(settings);

    match diesel::insert_into(appointments).values((type_id.eq(type_id2), entity_id.eq(entity_id2), appointment_date.eq(appointment_date2))).get_result(&connection){
        Ok(appointment) => Ok(appointment),
        Err(e) => {
            error!("Couldn't insert appointment: {}", e);
            Err(e)
        }
    }
}

pub fn remove_appointment(settings: &State<Settings>, ap_entity_id: uuid::Uuid) -> Result<(), diesel::result::Error>{
    use crate::schema::appointments::dsl::*;

    let connection = establish_connection(settings);

    match diesel::delete(appointments.filter(id.eq(ap_entity_id))).execute(&connection){
        Ok(_) => Ok(()),
        Err(e) => {
            error!("Coudn't remove appointment: {}", e);
            Err(e)
        }
    }
}

pub fn get_appointment_for_entity(settings: &State<Settings>, entity_id2: uuid::Uuid) -> Result<Vec<Appointment>, diesel::result::Error>{
    use crate::schema::appointments::dsl::*;

    let connection = establish_connection(settings);

    match appointments.filter(entity_id.eq(entity_id2)).load(&connection){
        Ok(appointment_list) => Ok(appointment_list),
        Err(e) => {
            error!("Couldn't get appointments for entity: {}", e);
            Err(e)
        }
    }
}

pub fn get_appointment_with_id(settings: &State<Settings>, ap_id: uuid::Uuid) -> Result<Appointment, diesel::result::Error>{
    use crate::schema::appointments::dsl::*;

    let connection = establish_connection(settings);

    match appointments.filter(id.eq(ap_id)).get_result(&connection){
        Ok(appointment) => Ok(appointment),
        Err(e) => {
            error!("Couldn't get appointment: {}", e);
            Err(e)
        }
    }
}