use crate::helper::settings::Settings;
use rocket::State;
use crate::database::model::event_requests::EventRequest;
use diesel::{RunQueryDsl, ExpressionMethods};
use crate::database::controller::connector::establish_connection;
use crate::diesel::QueryDsl;

pub fn add_event_request(settings: &State<Settings>, data: EventRequest) -> Result<EventRequest, diesel::result::Error>{
    use crate::schema::event_requests::dsl::*;

    let connection = establish_connection(settings);

    match diesel::insert_into(event_requests).values(data).get_result(&connection){
        Ok(er) => Ok(er),
        Err(e) => {
            error!("Couldn't create event request: {}", e);
            Err(e)
        }
    }
}

pub fn get_event_request(settings: &State<Settings>, request_id: uuid::Uuid) -> Result<Option<EventRequest>, diesel::result::Error>{
    use crate::diesel::OptionalExtension;
    use crate::schema::event_requests::dsl::*;

    let connection = establish_connection(settings);

    match event_requests.filter(entity_id.eq(request_id)).get_result(&connection).optional(){
        Ok(res) => Ok(res),
        Err(e) => {
            error!("Couldn't get event request: {}", e);
            Err(e)
        }
    }
}

pub fn get_event_requests(settings: &State<Settings>, limit: i64, offset: i64, state2: Option<i16>) -> Result<Vec<EventRequest>, diesel::result::Error>{
    use crate::schema::event_requests::dsl::*;

    let connection = establish_connection(settings);

    let mut query = event_requests.order(received.desc()).limit(limit).offset(offset).into_boxed();

    match state2{
        Some(state2) => {
            query = query.filter(state.eq(state2));
        }
        None => {}
    };

    match query.get_results(&connection){
        Ok(requestlist) => Ok(requestlist),
        Err(e) => {
            error!("Couldn't get event requests: {}", e);
            Err(e)
        }
    }
}

pub fn get_event_request_count(settings: &State<Settings>, state2: Option<i16>) -> Result<i64, diesel::result::Error>{
    use crate::schema::event_requests::dsl::*;
    use diesel::dsl::count;

    let connection = establish_connection(settings);

    let mut query = event_requests.select(count(entity_id)).into_boxed();

    match state2{
        Some(state2) => {query = query.filter(state.eq(state2))},
        None => {}
    };

    match query.get_result(&connection){
        Ok(requestcount) => Ok(requestcount),
        Err(e) => {
            error!("Couldn't get event request count: {}", e);
            Err(e)
        }
    }
}

pub fn change_event_request(settings: &State<Settings>, data: EventRequest) -> Result<EventRequest, diesel::result::Error>{
    use crate::schema::event_requests::dsl::*;

    let connection = establish_connection(settings);

    match diesel::update(event_requests.filter(entity_id.eq(data.entity_id))).set(data).get_result(&connection){
        Ok(er) => Ok(er),
        Err(e) => {
            error!("Couldn't update event request: {}", e);
            Err(e)
        }
    }
}