use diesel::{ExpressionMethods, QueryDsl, RunQueryDsl};
use rocket::State;

use crate::database::controller::connector::establish_connection;
use crate::database::model::events::EventUnitVehiclePosition;
use crate::Settings;

pub fn get_eu_vehicle_positions_for_template(settings: &State<Settings>, template_id: uuid::Uuid) -> Result<Vec<EventUnitVehiclePosition>, diesel::result::Error> {
    use crate::schema::eu_vehicle_positions::dsl;

    let connection = establish_connection(settings);
    match dsl::eu_vehicle_positions.filter(dsl::template_id.eq(template_id)).get_results(&connection) {
        Ok(pos) => Ok(pos),
        Err(e) => {
            error!("Couldn't get vehicle positions for template: {}", e);
            Err(e)
        }
    }
}

pub fn delete_eu_vehicle_position(settings: &State<Settings>, vehicle_position_id: uuid::Uuid) -> Result<(), diesel::result::Error> {
    use crate::schema::eu_vehicle_positions::dsl::*;

    let connection = establish_connection(settings);
    match diesel::delete(eu_vehicle_positions.filter(entity_id.eq(vehicle_position_id))).execute(&connection) {
        Ok(_) => Ok(()),
        Err(e) => {
            error!("Couldn't delete vehicle_position: {}", e);
            Err(e)
        }
    }
}

pub fn update_eu_vehicle_position(settings: &State<Settings>, vehicle_position_data: EventUnitVehiclePosition) -> Result<EventUnitVehiclePosition, diesel::result::Error> {
    use crate::schema::eu_vehicle_positions::dsl::*;

    let connection = establish_connection(settings);
    match diesel::update(eu_vehicle_positions.filter(entity_id.eq(vehicle_position_data.entity_id))).set(vehicle_position_data).get_result(&connection) {
        Ok(res) => Ok(res),
        Err(e) => {
            error!("Couldn't update vehicle position: {}", e);
            Err(e)
        }
    }
}