use rocket::State;
use crate::helper::settings::Settings;
use crate::database::controller::connector::establish_connection;

use crate::database::model::organisers::Organiser;

use diesel::dsl::count;
use crate::diesel::RunQueryDsl;

use diesel::{PgTextExpressionMethods, ExpressionMethods, BoolExpressionMethods};
use crate::diesel::QueryDsl;

pub fn get_organisers(settings: &State<Settings>, limit: i64, offset: i64, q: Option<String>) -> Result<Vec<Organiser>, diesel::result::Error>{
    use crate::schema::event_organisers::dsl::*;

    let connection = establish_connection(settings);

    let mut query = event_organisers.order(company.asc()).limit(limit).offset(offset).into_boxed();

    match q{
        Some(q) => {
            let splitted: Vec<&str> = q.split_whitespace().collect();
            if splitted.len() == 1{
                query = query.filter(company.ilike(format!("{}%", q)).or(firstname.ilike(format!("{}%", splitted[0]))).or(lastname.ilike(format!("{}%", splitted[0]))));
            }else if splitted.len() != 0 {
                query = query.filter(company.ilike(format!("{}%", q)).or(firstname.ilike(format!("{}%", splitted[0]))).and(lastname.ilike(format!("{}%", splitted[1]))));
            }
        }
        None => {}
    }

    match query.load(&connection){
        Ok(organisers) => Ok(organisers),
        Err(e) => {
            error!("Couldn't retrieve organisers: {}", e);
            Err(e)
        }
    }
}

pub fn get_organiser(settings: &State<Settings>, entity_id2: uuid::Uuid) -> Result<Organiser, diesel::result::Error>{
    use crate::schema::event_organisers::dsl::*;

    let connection = establish_connection(settings);

    match event_organisers.filter(entity_id.eq(entity_id2)).get_result(&connection){
        Ok(organiser) => Ok(organiser),
        Err(e) => {
            error!("Couldn't retrieve organiser: {}", e);
            Err(e)
        }
    }
}

pub fn get_organiser_count(settings: &State<Settings>) -> Result<i64, diesel::result::Error>{
    use crate::schema::event_organisers::dsl::*;

    let connection = establish_connection(settings);

    match event_organisers.select(count(entity_id)).first(&connection){
        Ok(count) => Ok(count),
        Err(e) => {
            error!("Couldn't count organisers: {}", e);
            Err(e)
        }
    }
}

pub fn add_organiser(settings: &State<Settings>, data: Organiser) -> Result<Organiser, diesel::result::Error>{
    use crate::schema::event_organisers::dsl::*;

    let connection = establish_connection(settings);

    match diesel::insert_into(event_organisers).values(data).get_result(&connection){
        Ok(org) => Ok(org),
        Err(e) => {
            error!("Couldn't create organiser: {}", e);
            Err(e)
        }
    }
}

pub fn change_organiser(settings: &State<Settings>, organiser: Organiser) -> Result<(), diesel::result::Error>{
    use crate::schema::event_organisers::dsl::*;

    let connection = establish_connection(settings);

    match diesel::update(event_organisers).filter(entity_id.eq(organiser.entity_id)).set(&organiser).execute(&connection){
        Ok(_) => Ok(()),
        Err(e) => {
            error!("Couldn't update organiser: {}", e);
            Err(e)
        }
    }
}