use crate::schema::event_requests;
use diesel::sql_types::{Uuid, Text, Nullable, Integer, SmallInt, Timestamp};
use chrono::NaiveDateTime;

#[derive(Queryable, Clone, Deserialize, Serialize, AsChangeset, Insertable, QueryableByName)]
#[table_name = "event_requests"]
#[changeset_options(treat_none_as_null = "true")]
#[primary_key(entity_id)]
pub struct EventRequest{
    #[sql_type = "Uuid"]
    pub(crate) entity_id: uuid::Uuid,
    #[sql_type = "Text"]
    pub(crate) name: String,
    #[sql_type = "Nullable<Text>"]
    pub(crate) timescale: Option<String>,
    #[sql_type = "Nullable<Text>"]
    pub(crate) site: Option<String>,
    #[sql_type = "Nullable<Uuid>"]
    pub(crate) organiser_id: Option<uuid::Uuid>,
    #[sql_type = "Nullable<Uuid>"]
    pub(crate) etype: Option<uuid::Uuid>,
    #[sql_type = "Nullable<Uuid>"]
    pub(crate) related_group: Option<uuid::Uuid>,
    #[sql_type = "Nullable<Text>"]
    pub(crate) contact_on_site_name: Option<String>,
    #[sql_type = "Nullable<Text>"]
    pub(crate) contact_on_site_phone: Option<String>,
    #[sql_type = "Nullable<Text>"]
    pub(crate) specials: Option<String>,
    #[sql_type = "Nullable<Text>"]
    pub(crate) requested: Option<String>,
    #[sql_type = "Nullable<Integer>"]
    pub(crate) expected_attendees: Option<i32>,
    #[sql_type = "Nullable<Integer>"]
    pub(crate) risk_potential: Option<i32>,
    #[sql_type = "SmallInt"]
    pub(crate) state: i16,
    #[sql_type = "Timestamp"]
    pub(crate) received: NaiveDateTime,
}