use crate::helper::sitebuilder::model::alerts::{Alert, AlertClass};
use crate::helper::sitebuilder::model::sidebar::{Communicator, EventBilling, EventManagement, MemberManagement, ResourceManagement, Settings, Sidebar, Summary};
use crate::modules::member_management::model::member::Member;

impl Alert {
    pub fn new(class: AlertClass, content: String) -> Alert {
        Alert { class, content }
    }
}

impl Sidebar {
    pub fn new(member: Member) -> Sidebar {
        Sidebar {
            firstname: member.firstname.clone(),
            lastname: member.lastname.clone(),
            summary: Summary {
                visible: member.has_permission(crate::permissions::modules::dashboard::VIEW.to_string()),
                active: false,
            },
            resource_management: ResourceManagement {
                visible: member.has_permission(crate::permissions::modules::resource_management::VIEW.to_string()),
                active: false,
            },
            member_management: MemberManagement {
                visible: member.has_permission(crate::permissions::modules::member_management::VIEW.to_string()),
                active: false,
            },
            event_management: EventManagement {
                visible: member.has_permission(crate::permissions::modules::event_management::VIEW.to_string()),
                active: false,
            },
            event_billing: EventBilling {
                visible: member.has_permission(crate::permissions::modules::event_billing::VIEW.to_string()),
                active: false,
            },
            communicator: Communicator {
                visible: member.has_permission(crate::permissions::modules::communicator::VIEW.to_string()),
                active: false,
            },
            settings: Settings {
                visible: member.has_permission(crate::permissions::modules::settings::VIEW.to_string()),
                active: false,
            }
        }
    }
}
