use diesel::{ExpressionMethods, RunQueryDsl};
use rocket::State;

use crate::database::controller::connector::establish_connection;
use crate::logger::entries::InsertableLogEntry;
use crate::schema::log::dsl::*;
use crate::Settings;

pub mod entries;

pub fn add_entry(entry: InsertableLogEntry, settings: &State<Settings>){
    let connection = establish_connection(settings);

    if let Err(err) = diesel::insert_into(crate::schema::log::table).values((action.eq(entry.action), affected_entity.eq(entry.affected_entity), causer.eq(entry.causer), details.eq(entry.details))).execute(&connection){
        error!("Couldn't add new application log entry: {}", err);
    }
}

pub enum LogActions{
    PasswordReset,
    BillingStateApproved,
}

impl LogActions{
    fn value(&self) -> String {
        match *self{
            LogActions::PasswordReset => String::from("password_reset"),
            LogActions::BillingStateApproved => String::from("billing_state_approved")
        }
    }
}