use rocket::http::Status;
use rocket::State;
use rocket_dyn_templates::Template;

use crate::database::controller::roles::get_roles;
use crate::helper::session_cookies::model::SessionCookie;
use crate::helper::settings::Settings;
use crate::helper::sitebuilder::model::general::{Footer, Header, Script, Stylesheet};
use crate::helper::sitebuilder::model::sidebar::Sidebar;
use crate::modules::admin_settings::permissions::SettingsModule;

#[get("/portal/settings/roles")]
pub fn settings_roles(cookie: SessionCookie, settings: &State<Settings>) -> Result<Template, Status> {
    let member = match cookie.member {
        //Unwraps member from cookie or send user to login if no member specified (user skipped member selection)
        Some(member) => member,
        None => return Err(Status::Unauthorized),
    };

    if !member.has_permission(
        crate::permissions::modules::event_management::events::VIEW.to_string(),
    ) {
        return Err(Status::Forbidden);
    }

    let header = Header {
        html_language: "de".to_string(),
        site_title: "Rollen".to_string(),
        stylesheets: vec![Stylesheet {
            path: "/css/errms.css".to_string(),
        }],
    };
    let footer = Footer {
        scripts: vec![Script {
            path: "/js/settings_roles.js".to_string(),
        }, Script {
            path: "/js/mini_searchbar.js".to_string(),
        }],
    };
    let mut sidebar = Sidebar::new(member.clone());
    sidebar.settings.active = true;

    let roles = match get_roles(settings){
        Ok(roles) => roles,
        Err(e) => {error!("Couldn't get roles: {}", e);return Err(Status::InternalServerError)}
    };

    let module = SettingsModule {
        header,
        footer,
        sidebar,
        caller: member.entity_id,
        roles
    };

    Ok(Template::render("module_settings_roles", module))
}