use rocket::State;
use crate::helper::settings::Settings;
use crate::helper::session_cookies::model::SessionCookie;
use rocket::serde::json::Json;
use crate::modules::api::model::api_outcome::{ApiErrorWrapper, ApiError};
use crate::modules::api::member_management::controller::parser::{parse_member_cookie, parse_uuid_string};
use crate::database::controller::entities::remove_entity;
use crate::helper::translate_diesel_error::translate_diesel;

#[delete("/api/events/<event_id>", format = "json")]
pub fn delete_event(
    settings: &State<Settings>,
    cookie: SessionCookie,
    event_id: String,
) -> Result<(), Json<ApiErrorWrapper>> {
    let caller = parse_member_cookie(cookie.member)?;
    let event_id = parse_uuid_string(event_id)?;

    if !caller.has_permission(crate::permissions::modules::event_management::events::DELETE.to_string()) {
        return Err(Json(
            ApiError::new(403, "Keine Berechtigung Einsatz zu löschen!".to_string()).to_wrapper(),
        ));
    }

    match remove_entity(settings, event_id){
        Ok(_) => {}
        Err(e) => return Err(translate_diesel(e)),
    }

    Ok(())
}