use rocket::serde::json::Json;
use rocket::State;

use crate::database::controller::entities::remove_entity;
use crate::database::controller::events::templates::vehicle_positions::delete_eu_vehicle_position;
use crate::helper::session_cookies::model::SessionCookie;
use crate::helper::settings::Settings;
use crate::helper::translate_diesel_error::translate_diesel;
use crate::modules::api::member_management::controller::parser::{parse_member_cookie, parse_uuid_string};
use crate::modules::api::model::api_outcome::{ApiError, ApiErrorWrapper};

#[delete("/api/events/units/positions", format = "json", data = "<delete_list>")]
pub fn delete_event_unit_positions(
    settings: &State<Settings>,
    cookie: SessionCookie,
    delete_list: Json<Vec<uuid::Uuid>>,
) -> Result<(), Json<ApiErrorWrapper>> {
    let caller = parse_member_cookie(cookie.member)?;
    if !caller.has_permission(crate::permissions::modules::event_management::events::EDIT.to_string()) {
        return Err(Json(
            ApiError::new(403, "Keine Berechtigung, Positionen zu löschen!".to_string()).to_wrapper(),
        ));
    }

    let delete_list = delete_list.into_inner();

    for position in delete_list {
        match remove_entity(settings, position){
            Ok(_) => {}
            Err(e) => return Err(translate_diesel(e)),
        }
    }

    Ok(())
}

#[delete("/api/events/units/vehicle_positions/<vehicle_position_id>", format = "json")]
pub fn delete_event_unit_vehicle_positions(
    settings: &State<Settings>,
    cookie: SessionCookie,
    vehicle_position_id: String,
) -> Result<(), Json<ApiErrorWrapper>> {
    let caller = parse_member_cookie(cookie.member)?;
    if !caller.has_permission(crate::permissions::modules::event_management::events::EDIT.to_string()) {
        return Err(Json(
            ApiError::new(403, "Keine Berechtigung, Positionen zu löschen!".to_string()).to_wrapper(),
        ));
    }

    match delete_eu_vehicle_position(settings, parse_uuid_string(vehicle_position_id)?) {
        Ok(()) => Ok(()),
        Err(e) => return Err(translate_diesel(e))
    }
}