use rocket::serde::json::Json;
use rocket::State;

use crate::database::controller::entities::remove_entity;
use crate::helper::session_cookies::model::SessionCookie;
use crate::helper::settings::Settings;
use crate::helper::translate_diesel_error::translate_diesel;
use crate::modules::api::member_management::controller::parser::{parse_member_cookie, parse_uuid_string};
use crate::modules::api::model::api_outcome::{ApiError, ApiErrorWrapper};

#[delete("/api/events/instances/<instance_id>", format = "json")]
pub fn delete_instance(
    settings: &State<Settings>,
    cookie: SessionCookie,
    instance_id: String,
) -> Result<(), Json<ApiErrorWrapper>> {
    let caller = parse_member_cookie(cookie.member)?;
    if !caller.has_permission(crate::permissions::modules::event_management::events::EDIT.to_string()) {
        return Err(Json(
            ApiError::new(403, "Keine Berechtigung Einsätze zu bearbeiten!".to_string()).to_wrapper(),
        ));
    }

    match remove_entity(settings, parse_uuid_string(instance_id)?) {
        Ok(_) => Ok(()),
        Err(e) => return Err(translate_diesel(e))
    }
}

/// Delete position_instance with specified position_instance_id
/// Returns Ok() (200) if deleted successfully, otherwise ApiError
#[delete("/api/events/position_instances/<position_instance_id>", format = "json")]
pub fn delete_position_instance(
    settings: &State<Settings>,
    cookie: SessionCookie,
    position_instance_id: String,
) -> Result<(), Json<ApiErrorWrapper>> {
    let caller = parse_member_cookie(cookie.member)?;
    if !caller.has_permission(crate::permissions::modules::event_management::events::EDIT.to_string()) {
        return Err(Json(
            ApiError::new(403, "Keine Berechtigung Einsätze zu bearbeiten!".to_string()).to_wrapper(),
        ));
    }

    match crate::database::controller::events::instances::instance_positions::delete(settings, parse_uuid_string(position_instance_id)?) {
        Ok(_) => Ok(()),
        Err(e) => return Err(translate_diesel(e))
    }
}