use rocket::State;
use crate::helper::settings::Settings;
use crate::helper::session_cookies::model::SessionCookie;
use rocket::serde::json::Json;
use crate::modules::api::model::api_outcome::ApiErrorWrapper;
use crate::modules::api::member_management::controller::parser::{parse_member_cookie, parse_uuid_string};
use crate::database::model::events::EventType;
use crate::database::controller::events::{get_event_types, get_event_type};
use crate::helper::translate_diesel_error::translate_diesel;

#[get("/api/events/types", format = "json")]
pub fn read_event_types(
    settings: &State<Settings>,
    cookie: SessionCookie,
) -> Result<Json<Vec<EventType>>, Json<ApiErrorWrapper>> {
    let _caller = parse_member_cookie(cookie.member)?;

    match get_event_types(settings){
        Ok(types) => Ok(Json(types)),
        Err(e) => Err(translate_diesel(e))
    }
}

#[get("/api/events/types/<type_id>", format = "json")]
pub fn read_event_type(
    settings: &State<Settings>,
    cookie: SessionCookie,
    type_id: String,
) -> Result<Json<EventType>, Json<ApiErrorWrapper>> {
    let _caller = parse_member_cookie(cookie.member)?;

    let etype = parse_uuid_string(type_id)?;
    match get_event_type(settings, etype){
        Ok(etype) => Ok(Json(etype)),
        Err(e) => Err(translate_diesel(e))
    }
}