use crate::helper::session_cookies::model::SessionCookie;
use crate::helper::settings::Settings;
use rocket::State;
use rocket::serde::json::Json;
use crate::database::model::member_licenses::LicenseCategory;
use crate::modules::api::model::api_outcome::{ApiErrorWrapper};
use crate::modules::api::member_management::controller::parser::parse_member_cookie;
use crate::database::controller::member_licenses::get_license_categories;
use crate::helper::translate_diesel_error::translate_diesel;

#[get(
"/api/members/license_categories",
format = "json"
)]
pub fn read_license_categories(
    cookie: SessionCookie,
    settings: &State<Settings>,
) -> Result<Json<Vec<LicenseCategory>>, Json<ApiErrorWrapper>> {
    let _caller = parse_member_cookie(cookie.member)?;

    match get_license_categories(settings){
        Ok(licenses) => Ok(Json(licenses)),
        Err(e) => Err(translate_diesel(e))
    }
}