use rocket::serde::json::Json;
use rocket::State;

use crate::helper::session_cookies::model::SessionCookie;
use crate::helper::translate_diesel_error::translate_diesel;
use crate::modules::api::member_management::controller::parser::{parse_member_cookie, parse_uuid_string};
use crate::modules::api::model::api_outcome::{ApiError, ApiErrorWrapper};
use crate::Settings;

#[delete("/api/personnel_billing/<position_instance_id>", format = "json")]
pub fn delete_position_instance(
    settings: &State<Settings>,
    cookie: SessionCookie,
    position_instance_id: String,
) -> Result<(), Json<ApiErrorWrapper>> {
    let caller = parse_member_cookie(cookie.member)?;
    if !caller.has_permission(crate::permissions::modules::event_billing::personnel::EDIT.to_string()) {
        return Err(Json(
            ApiError::new(403, "Keine Berechtigung Abrechnung zu bearbeiten!".to_string()).to_wrapper(),
        ));
    }

    match crate::database::controller::billing::personnel_billing::remove(settings, parse_uuid_string(position_instance_id)?) {
        Ok(_) => Ok(()),
        Err(e) => return Err(translate_diesel(e))
    }
}