use rocket::State;
use crate::helper::settings::Settings;
use crate::helper::session_cookies::model::SessionCookie;
use rocket::serde::json::Json;
use crate::modules::api::resources::vehicles::create::CreateVehicleData;
use crate::modules::api::model::api_outcome::{ApiErrorWrapper, ApiError};
use crate::modules::api::member_management::controller::parser::{parse_member_cookie, parse_uuid_string};
use crate::database::controller::vehicles::change_vehicle;
use crate::database::model::vehicles::Vehicle;

use crate::helper::translate_diesel_error::translate_diesel;


use crate::database::controller::members::check_access_to_resource;

#[put("/api/resources/vehicles/<vehicle_id>", format = "json", data = "<change_vehicle_data>")]
pub fn update_vehicle(
    settings: &State<Settings>,
    cookie: SessionCookie,
    change_vehicle_data: Json<CreateVehicleData>,
    vehicle_id: String,
) -> Result<(), Json<ApiErrorWrapper>> {
    let caller = parse_member_cookie(cookie.clone().member)?;
    let vehicle_id = parse_uuid_string(vehicle_id)?;

    if !check_access_to_resource(settings, caller.entity_id, vehicle_id, crate::permissions::modules::resource_management::vehicles::core::EDIT){
        return Err(Json(
            ApiError::new(403, "Keine Berechtigung, Fahrzeuge zu ändern!".to_string()).to_wrapper(),
        ));
    }

    let vehicle_category = match change_vehicle_data.vehicle_category.clone(){
        Some(c) => {
            Some(parse_uuid_string(c)?)
        },
        None => None
    };


    let vehicle = Vehicle{
        entity_id: vehicle_id,
        identifier: change_vehicle_data.identifier.clone(),
        numberplate: change_vehicle_data.numberplate.clone(),
        description: change_vehicle_data.description.clone(),
        is_operational: change_vehicle_data.is_operational,
        admissible_total_weight: change_vehicle_data.admissible_total_weight,
        required_license: None,
        vehicle_category,
    };

    match change_vehicle(settings, vehicle){
        Ok(_) => Ok(()),
        Err(e) => Err(translate_diesel(e))
    }

}