use crate::helper::session_cookies::model::SessionCookie;
use rocket::State;
use crate::helper::settings::Settings;

use rocket::http::Status;
use crate::helper::sitebuilder::model::general::{Header, Stylesheet, Footer, Script};
use crate::helper::sitebuilder::model::sidebar::Sidebar;
use crate::modules::event_management::eventlist::EventTemplates;
use rocket_dyn_templates::Template;

#[get("/portal/em/eu_templates")]
pub fn event_unit_templates(cookie: SessionCookie, _settings: &State<Settings>) -> Result<Template, Status> {
    let member = match cookie.member {
        //Unwraps member from cookie or send user to login if no member specified (user skipped member selection)
        Some(member) => member,
        None => return Err(Status::Unauthorized),
    };

    if !member.has_permission(
        crate::permissions::modules::event_management::events::EDIT.to_string(),
    ) {
        return Err(Status::Forbidden);
    }

    let header = Header {
        html_language: "de".to_string(),
        site_title: "Vorlagen".to_string(),
        stylesheets: vec![Stylesheet {
            path: "/css/errms.css".to_string(),
        }],
    };
    let footer = Footer {
        scripts: vec![Script {
            path: "/js/em_eu_templates.js".to_string(),
        }, Script {
            path: "/js/mini_searchbar.js".to_string(),
        }, Script {
            path: "/js/pagination.js".to_string(),
        }],
    };
    let mut sidebar = Sidebar::new(member.clone());
    sidebar.event_management.active = true;

    let eventlist = EventTemplates {
        header,
        footer,
        sidebar,
        caller: member.entity_id
    };

    Ok(Template::render("module_em_event_unit_templates", eventlist))
}
