use crate::database::model::addresses::Address;
use crate::database::model::member_communication::CommunicationTargetCreate;
use crate::helper::sitebuilder::model::general::{Footer, Header};
use crate::helper::sitebuilder::model::sidebar::Sidebar;
use crate::modules::member_management::model::groups::Group;
use crate::modules::member_management::model::licenses::MemberLicense;
use crate::modules::member_management::model::member::{Member, MemberWithAccess};
use crate::modules::member_management::model::qualifications::QualificationCategory;
use std::collections::HashMap;
use crate::database::controller::groups::GroupMembership;
use crate::database::model::units::Unit;
use crate::modules::member_management::model::login::Login;
use crate::database::model::groups::RawGroup;

#[derive(Serialize)]
pub struct MemberModuleSelection {
    pub header: Header,
    pub footer: Footer,
    pub sidebar: Sidebar,
    pub member_list: Vec<MemberWithAccess>,
    pub group_list: HashMap<uuid::Uuid, Group>,
    pub previous_page : Option<i64>,
    pub next_page: Option<i64>,
    pub last_page: i64,
    pub search_parameters: String,
    pub page: i64,
}

#[derive(Serialize)]
pub struct EmptyMemberModuleProfile {
    pub header: Header,
    pub footer: Footer,
    pub sidebar: Sidebar,
}

#[derive(Serialize)]
pub struct MemberModuleProfile {
    pub header: Header,
    pub footer: Footer,
    pub member: Member,
    pub member_age: Option<i32>,
    pub address: Option<Address>,
    pub licenses: Vec<MemberLicense>,
    pub qualification_categories: Vec<QualificationCategory>,
    pub communication_targets: Vec<CommunicationTargetCreate>,
    pub readonly: bool,
    pub sidebar: Sidebar,
    pub caller_permissions: Vec<String>,
    pub units: Vec<Unit>,
    pub groups: Vec<GroupMembership>,
    pub login: Login,
}

#[derive(Serialize)]
pub struct MemberModuleCreate{
    pub header: Header,
    pub footer: Footer,
    pub sidebar: Sidebar,
    pub groups: Vec<RawGroup>
}