use crate::helper::session_cookies::model::SessionCookie;
use crate::helper::settings::Settings;
use rocket::http::Status;
use rocket::State;
use rocket_dyn_templates::Template;
use crate::helper::sitebuilder::model::sidebar::Sidebar;
use crate::helper::sitebuilder::model::general::{Footer, Header, Stylesheet, Script};

#[derive(Serialize)]
pub struct MemberSelection{
    pub(crate) header: Header,
    pub(crate) footer: Footer,
    pub(crate) sidebar: Sidebar,
}

#[get("/portal/mm")]
pub fn member_management_selection(
    cookie: SessionCookie,
    settings: &State<Settings>,
) -> Result<Template, Status> {
    let member = match cookie.member {
        //Unwraps member from cookie or send user to login if no member specified (user skipped member selection)
        Some(member) => member,
        None => return Err(Status::Unauthorized),
    };

    if !member.has_permission(crate::permissions::modules::member_management::list::VIEW.to_string()) {
        return Err(Status::Forbidden);
    }

    let header = Header {
        html_language: "de".to_string(),
        site_title: "Member List".to_string(),
        stylesheets: vec![Stylesheet {
            path: "/css/errms.css".to_string(),
        }],
    };
    let footer = Footer {
        scripts: vec![Script {
            path: "/js/member_management_delete_member.js".to_string(),
        }, Script {
            path: "/js/mm_list.js".to_string(),
        },Script {
            path: "/js/pagination.js".to_string(),
        }],
    };
    let mut sidebar = Sidebar::new(member.clone());
    sidebar.member_management.active = true;

    let ms = MemberSelection{
        header,
        footer,
        sidebar
    };

    Ok(Template::render(
        "module_member_management_selection_new",
        ms),
    )
}
