use crate::helper::session_cookies::model::SessionCookie;
use crate::helper::settings::Settings;
use rocket::http::Status;
use rocket::State;
use rocket_dyn_templates::Template;
use crate::helper::sitebuilder::model::general::{Header, Stylesheet, Footer, Script};

#[derive(Serialize)]
pub struct PrintEventNote{
    pub header: Header,
    pub footer: Footer,
}

#[get("/portal/print/event_note?<event_id>")]
pub fn print_event_note(cookie: SessionCookie, settings: &State<Settings>, event_id: String) -> Result<Template, Status> {
    let member = match cookie.member {
        //Unwraps member from cookie or send user to login if no member specified (user skipped member selection)
        Some(member) => member,
        None => return Err(Status::Unauthorized),
    };

    if !member.has_permission(
        crate::permissions::modules::event_management::events::VIEW.to_string(),
    ) {
        return Err(Status::Forbidden);
    }

    let header = Header {
        html_language: "de".to_string(),
        site_title: "Einsatzauftrag".to_string(),
        stylesheets: vec![Stylesheet {
            path: "/css/errms.css".to_string(),
        }],
    };
    let footer = Footer {
        scripts: vec![Script {
            path: "/js/print.js".to_string(),
        }],
    };

    let pen : PrintEventNote = PrintEventNote{
        header,
        footer
    };

    Ok(Template::render("print_event_note", pen))
}
