use crate::helper::session_cookies::model::SessionCookie;




use rocket::http::Status;
use crate::helper::sitebuilder::model::general::{Header, Footer, Script, Stylesheet};
use crate::helper::sitebuilder::model::sidebar::Sidebar;
use rocket_dyn_templates::Template;

#[derive(Serialize)]
pub struct VehicleList{
    pub header: Header,
    pub footer: Footer,
    pub sidebar: Sidebar,
}

#[get("/portal/rm/vehicle_list")]
pub fn vehicle_list(
    cookie: SessionCookie,
) -> Result<Template, Status> {
    let member = match cookie.member {
        //Unwraps member from cookie or send user to login if no member specified (user skipped member selection)
        Some(member) => member,
        None => return Err(Status::Unauthorized),
    };

    if !member.has_permission(crate::permissions::modules::resource_management::vehicles::list::VIEW.to_string()) {
        return Err(Status::Forbidden);
    }

    let header = Header {
        html_language: "de".to_string(),
        site_title: "Fahrzeuge".to_string(),
        stylesheets: vec![Stylesheet {
            path: "/css/errms.css".to_string(),
        }],
    };
    let footer = Footer {
        scripts: vec![Script {
            path: "/js/resource_management_vehicle_list.js".to_string(),
        }, Script{
            path: "/js/scheduler.js".to_string()
        }],
    };
    let mut sidebar = Sidebar::new(member.clone());
    sidebar.resource_management.active = true;

    let vehicle_list = VehicleList{
        header,
        footer,
        sidebar
    };

    Ok(Template::render("module_resource_management_vehicle_list", vehicle_list))
}
